/*
 * File XMLDemo.java
 * Authors: me and he
 */

package ija.xml;

import java.util.*;
import java.io.*;
import org.dom4j.*;
import org.dom4j.dom.*;
import org.dom4j.io.*;

/**
 * Class XMLDemo shows basic oprations with XML documents.
 * @author Radek Koci
 */
public class XMLDemo {

    /**
     * Represents a tree of an open XML based document.
     */
    protected Document doc;

    /**
     * Iterates though all elements in the XML based document. 
     * The document tree is accessible via the object variable {@link #doc}.
     */
    public void iterateRootChildren() {
	Element root = this.doc.getRootElement();
	Iterator elementIterator = root.elementIterator();
	while(elementIterator.hasNext()){
	    Element element = (Element)elementIterator.next();
	    System.out.println(element.getName());
	    Iterator innerIt = element.elementIterator();
	    while(innerIt.hasNext()){
		Element innerElement = (Element) innerIt.next();
		System.out.println(element.getName() + "/" + innerElement.getName() + ":" + innerElement.getText());
	    }
	}
    }   

    /**
     * Opens a XML based file and creates its tree representation.
     * @param aFile XML based file to open
     * @throws DocumentException
     * @see #iterateRootChildren() 
     */
    public void parseWithSAX(File aFile) throws DocumentException {
	SAXReader xmlReader = new SAXReader();
        this.doc = xmlReader.read(aFile);
    }

    /**
     * Iterates though all elements in the XML based document. 
     * @deprecated Use the method {@link #iterateRootChildren()} instead.
     */
    public void browseRootChildren() {
	XPath xpathSelector = DocumentHelper.createXPath("//author[@location='UK']/full_name");
	List results = xpathSelector.selectNodes(doc);
	for ( Iterator iter = results.iterator(); iter.hasNext(); ) {
	    Element element = (Element) iter.next();
	    System.out.println(element.getText());
	}
										     
    }

    /**
     * Creates a new XML based file.
     * @param file a name of XML based file to be created
     * @throws IOException
     */
    public void newJDomDocument(String file) throws IOException {
	doc = DocumentHelper.createDocument();
	
        Element root = doc.addElement( "root" );
    
        Element author3 = root.addElement( "author" )
	    .addAttribute( "name", "Sam" )
            .addAttribute( "location", "UK" );
        author3.addElement( "year" )
	    .addText( "1965" );
        author3.addElement( "full_name" )
	    .addText( "Sam Penguin" );
			  
        Element author2 = root.addElement( "author" )
	    .addAttribute( "name", "Toby" )
            .addAttribute( "location", "Germany" );
        author2.addElement( "year" )
	    .addText( "1957" );
        author2.addElement( "full_name" )
	    .addText( "Tobias Rademacher" );
			  

	Element author1 = root.addElement( "author" )
	    .addAttribute( "name", "James" )
	    .addAttribute( "location", "UK" );
        author1.addElement( "year" )
	    .addText( "1963" );
        author1.addElement( "full_name" )
	    .addText( "James Strachan" );	
  
	OutputFormat outformat = OutputFormat.createPrettyPrint();
	Writer out = new OutputStreamWriter(new FileOutputStream(file));
	XMLWriter writer = new XMLWriter(out, outformat);
	writer.write(doc);
	writer.flush();	
    }

    /**
     * Creates a new XML based file.
     * @deprecated Use the method {@link #newJDomDocument(java.lang.String)} instead.
     */
    public void newDocument() throws IOException {
	doc = new DOMDocument();
	DOMElement el = new DOMElement("blah");
	doc.add(el);
	
	DOMElement el2 = new DOMElement("name");
	el2.setAttribute("id","28");
	el2.setText("Ferda");
	el.add(el2);
	
	Writer out = new OutputStreamWriter(new FileOutputStream("p.xml"));
	XMLWriter writer = new XMLWriter(out);
	writer.write(doc);
	writer.flush();	
    }

	        
    public static void main(String[] args) {
	try {
    	    XMLDemo demo = new XMLDemo();    
	    demo.newJDomDocument("demo.xml");
	    demo.parseWithSAX(new File("demo.xml"));
	    demo.iterateRootChildren();

//	    demo.browseRootChildren(); // needs jaxen and saxpath 
//	    demo.newDocument();
	} 
	catch (DocumentException ex) {
	    ex.printStackTrace();
	}
	catch (IOException ex) {
	    ex.printStackTrace();
	}
    }
}
